__all__ = [
    'getCredentialsData',
    'decode_base64'
]

import os
import sys
import argparse
import requests
import pickle
import base64
from pathlib import Path
from lps_maestro.constants import *
import re

def getCredentialsData ():
  home = str(Path.home())
  try:
    f = open("{}/{}".format(home, CREDENTIALS_FILE), "r")
    if f.mode == 'r':
      content = f.read()
      f.close()
      return content
  except FileNotFoundError:
    print ("Please authenticate first")
    return False

def decode_base64(data, altchars=b'+/'):
    """Decode base64, padding being optional.

    :param data: Base64 data as an ASCII byte string
    :returns: The decoded byte string.

    """
    data = re.sub(rb'[^a-zA-Z0-9%s]+' % altchars, b'', data)  # normalize
    missing_padding = len(data) % 4
    if missing_padding:
        data += b'='* (4 - missing_padding)
    return base64.b64decode(data, altchars)



__all__ = [
  'task'
]

import os
import sys
import argparse
import requests
import pickle
import base64
from pathlib import Path
from lps_maestro.utils import getCredentialsData, decode_base64
from lps_maestro.constants import *

class Task ():
  def create( self, taskname,
                    dataFile,
                    configFile,
                    execCommand,
                    containerImage,
                    queue='cpu_small',
                    secondaryDS=None,
                    et=None,
                    eta=None,
                    dry_run=False):

    if taskname.split('.')[0] != 'user':
      print( 'The task name must start with "user.<username>.taskname."')
      raise BaseException
    username = taskname.split('.')[1]

    credentials = getCredentialsData()
    if credentials == False:
      return

    if dry_run:
      print ("Please disable dry run.")
      return

    data = {
      'username'              : username,
      'taskname'              : taskname,
      'configFile'            : configFile,
      'dataFile'              : dataFile,
      'containerImage'        : containerImage,
      'secondaryDS'           : secondaryDS if secondaryDS else '',
      'execCommand'           : execCommand,
      'et'                    : et if et else 0,
      'eta'                   : eta if eta else 0,
      'queue'                 : queue,
      'credentials'           : credentials
    }

    try:
      r = requests.post(url='http://146.164.147.170:5020/create-task', data=data)
      print (r.text)
    except requests.exceptions.ConnectionError:
      print ("Failed to connect to LPS Cluster.")

  def delete( self, taskname ):

    if taskname.split('.')[0] != 'user':
      print( 'The task name must start with "user.<username>.taskname."')
      raise BaseException
    username = taskname.split('.')[1]

    credentials = getCredentialsData()
    if credentials == False:
      return

    data = {
      'username':username,
      'taskname':taskname,
      'credentials':credentials
    }

    try:
      r = requests.post(url='http://146.164.147.170:5020/delete-task', data=data)
      print (r.text)
    except requests.exceptions.ConnectionError:
      print ("Failed to connect to LPS Cluster.")

  def retry( self, taskname ):

    if taskname.split('.')[0] != 'user':
      print( 'The task name must start with "user.<username>.taskname."')
      raise BaseException
    username = taskname.split('.')[1]

    credentials = getCredentialsData()
    if credentials == False:
      return

    data = {
      'username':username,
      'taskname':taskname,
      'credentials':credentials
    }

    try:
      r = requests.post(url='http://146.164.147.170:5020/retry-task', data=data)
      print (r.text)
    except requests.exceptions.ConnectionError:
      print ("Failed to connect to LPS Cluster.")

  def list( self, username, cli=False ):

    credentials = getCredentialsData()
    if credentials == False:
      return

    data = {
      'username':username,
      'credentials':credentials
    }
    try:
      if cli:
        r = requests.post(url='http://146.164.147.170:5020/list-tasks', data=data)
        print (r.json()['message'])
      else:
        r = requests.post(url='http://146.164.147.170:5020/list-tasks-py', data=data)
        try:
          return r.json()
        except:
          pickled_response = r.content.decode('utf-8')
          df = pickle.loads(decode_base64(pickled_response.encode()))
          return df
    except requests.exceptions.ConnectionError:
      print ("Failed to connect to LPS Cluster.")

  def kill( self, username, taskname ):

    if taskname.split('.')[0] != 'user':
      print( 'The task name must start with "user.<username>.taskname."')
      raise BaseException

    credentials = getCredentialsData()
    if credentials == False:
      return

    data = {
      'username':username,
      'taskname':taskname,
      'credentials':credentials
    }

    try:
      r = requests.post(url='http://146.164.147.170:5020/kill-task', data=data)
      print (r.text)
    except requests.exceptions.ConnectionError:
      print ("Failed to connect to LPS Cluster.")

task = Task()


__all__ = [
  'authenticate'
]

import os
import sys
import argparse
import requests
from hashlib import sha256, md5
import pickle
import base64
from pathlib import Path
from lps_maestro.utils import getCredentialsData
from lps_maestro.constants import *

class Authenticate ():

  def __init__(self):
    self.__class__ = type(self.__class__.__name__, (self.__class__,), {})
    self.__class__.__call__ = self.authenticate

  def hashPw (self, password):
    m = md5()
    m.update(password.encode('utf-8'))
    return m.hexdigest()

  def authenticate (self, username, password):
    print ("Trying to connect...")
    data = {
      'username':username,
      'password':password
    }
    try:
      r = requests.post(url='http://146.164.147.170:5020/authenticate', data=data)
      print (r.text)
      if (r.json()['error_code'] == 200):
        home = str(Path.home())
        f = open("{}/{}".format(home, CREDENTIALS_FILE), "wb+")
        f.write("${}${}".format(username, r.json()['token']).encode('utf-8'))
        f.close()
    except requests.exceptions.ConnectionError:
      print ("Failed to connect to LPS Cluster.")

authenticate = Authenticate()


from setuptools import setup, find_packages
from setuptools.extension import Extension
import subprocess
import sys

def install(package):
  subprocess.check_call([sys.executable, "-m", "pip", "install", package])

try:
  from Cython.Build import cythonize
except:
  install('cython')
  from Cython.Build import cythonize

with open("README.md", "r") as fh:
  long_description = fh.read()

extensions = [
  Extension(
    "lps_maestro.*",
    ["lps_maestro/*.py"],
  )
]

setup(
  name = 'lps-maestro',
  version = '0.4.2',
  license='GPL-3.0',
  description = 'Command line interface for the LPS Cluster @ UFRJ.',
  long_description = long_description,
  long_description_content_type="text/markdown",
  packages=['lps_maestro'],
  author = 'Gabriel Gazola Milan',
  author_email = 'gabriel.gazola@poli.ufrj.br',
  url = 'https://github.com/gabriel-milan/maestro',
  keywords = ['cli', 'cluster', 'command-line interface', 'computer grid', 'lps', 'ufrj'],
  install_requires=[
    'requests'
  ],
  classifiers=[
    'Development Status :: 4 - Beta',
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Libraries :: Python Modules',
    'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
  ],
  ext_modules = cythonize(extensions)
)
